@@ -4,10 +4,10 @@ use vortex_array::arrays::PrimitiveArray;
 use vortex_array::patches::Patches;
 use vortex_array::stats::{ArrayStats, StatsSetRef};
 use vortex_array::validity::Validity;
-use vortex_array::vtable::{EncodingVTable, StatisticsVTable, VTableRef};
+use vortex_array::vtable::VTableRef;
 use vortex_array::{
     Array, ArrayCanonicalImpl, ArrayImpl, ArrayRef, ArrayStatisticsImpl, ArrayValidityImpl,
-    Canonical, Encoding, EncodingId, SerdeMetadata, ToCanonical,
+    Canonical, Encoding, SerdeMetadata, ToCanonical,
 };
 use vortex_buffer::Buffer;
 use vortex_dtype::{DType, PType};
@@ -34,12 +34,6 @@ impl Encoding for ALPRDEncoding {
     type Metadata = SerdeMetadata<ALPRDMetadata>;
 }
 
-impl EncodingVTable for ALPRDEncoding {
-    fn id(&self) -> EncodingId {
-        EncodingId::new_ref("vortex.alprd")
-    }
-}
-
 impl ALPRDArray {
     pub fn try_new(
         dtype: DType,
@@ -156,6 +150,26 @@ impl ArrayImpl for ALPRDArray {
     fn _vtable(&self) -> VTableRef {
         VTableRef::new_ref(&ALPRDEncoding)
     }
+
+    fn _with_children(&self, children: &[ArrayRef]) -> VortexResult<Self> {
+        let left_parts = children[0].clone();
+        let right_parts = children[1].clone();
+
+        let left_part_patches = self.left_parts_patches().map(|existing| {
+            let indices = children[2].clone();
+            let values = children[3].clone();
+            Patches::new(existing.array_len(), existing.offset(), indices, values)
+        });
+
+        ALPRDArray::try_new(
+            self.dtype().clone(),
+            left_parts,
+            self.left_parts_dictionary().clone(),
+            right_parts,
+            self.right_bit_width(),
+            left_part_patches,
+        )
+    }
 }
 
 impl ArrayCanonicalImpl for ALPRDArray {
@@ -218,8 +232,6 @@ impl ArrayValidityImpl for ALPRDArray {
     }
 }
 
-impl StatisticsVTable<&ALPRDArray> for ALPRDEncoding {}
-
 #[cfg(test)]
 mod test {
     use rstest::rstest;
