@@ -1,11 +1,11 @@
-use vortex_array::compute::{FilterFn, filter};
+use vortex_array::compute::{FilterKernel, filter};
 use vortex_array::{Array, ArrayRef};
 use vortex_error::VortexResult;
 use vortex_mask::Mask;
 
 use crate::{ALPRDArray, ALPRDEncoding};
 
-impl FilterFn<&ALPRDArray> for ALPRDEncoding {
+impl FilterKernel for ALPRDEncoding {
     fn filter(&self, array: &ALPRDArray, mask: &Mask) -> VortexResult<ArrayRef> {
         let left_parts_exceptions = array
             .left_parts_patches()
