@@ -1,29 +1,29 @@
-use vortex_array::array::ConstantArray;
-use vortex_array::compute::{compare, CompareFn, Operator};
-use vortex_array::{Array, IntoArray, IntoArrayVariant};
+use vortex_array::arrays::ConstantArray;
+use vortex_array::compute::{CompareFn, Operator, compare};
+use vortex_array::{Array, ArrayRef, ToCanonical};
 use vortex_error::VortexResult;
 
 use crate::compress::runend_decode_bools;
 use crate::{RunEndArray, RunEndEncoding};
 
-impl CompareFn<RunEndArray> for RunEndEncoding {
+impl CompareFn<&RunEndArray> for RunEndEncoding {
     fn compare(
         &self,
         lhs: &RunEndArray,
-        rhs: &Array,
+        rhs: &dyn Array,
         operator: Operator,
-    ) -> VortexResult<Option<Array>> {
+    ) -> VortexResult<Option<ArrayRef>> {
         // If the RHS is constant, then we just need to compare against our encoded values.
         if let Some(const_scalar) = rhs.as_constant() {
             return compare(
                 lhs.values(),
-                ConstantArray::new(const_scalar, lhs.values().len()),
+                &ConstantArray::new(const_scalar, lhs.values().len()),
                 operator,
             )
             .and_then(|values| {
                 runend_decode_bools(
-                    lhs.ends().into_primitive()?,
-                    values.into_bool()?,
+                    lhs.ends().to_primitive()?,
+                    values.to_bool()?,
                     lhs.offset(),
                     lhs.len(),
                 )
@@ -38,9 +38,9 @@ impl CompareFn<RunEndArray> for RunEndEncoding {
 }
 #[cfg(test)]
 mod test {
-    use vortex_array::array::{BooleanBuffer, ConstantArray, PrimitiveArray};
-    use vortex_array::compute::{compare, Operator};
-    use vortex_array::{IntoArray, IntoArrayVariant};
+    use vortex_array::arrays::{BooleanBuffer, ConstantArray, PrimitiveArray};
+    use vortex_array::compute::{Operator, compare};
+    use vortex_array::{Array, ToCanonical};
 
     use crate::RunEndArray;
 
@@ -54,11 +54,11 @@ mod test {
     #[test]
     fn compare_run_end() {
         let arr = ree_array();
-        let res = compare(arr, ConstantArray::new(5, 12), Operator::Eq).unwrap();
-        let res_canon = res.into_bool().unwrap();
+        let res = compare(&arr, &ConstantArray::new(5, 12), Operator::Eq).unwrap();
+        let res_canon = res.to_bool().unwrap();
         assert_eq!(
             res_canon.boolean_buffer(),
-            BooleanBuffer::from(vec![
+            &BooleanBuffer::from(vec![
                 false, false, false, false, false, false, false, false, true, true, true, true
             ])
         );
