@@ -1,11 +1,11 @@
 use pyo3::{Bound, PyRef, PyResult, pyclass, pymethods};
 use vortex::arrays::ConstantEncoding;
 
-use crate::arrays::{AsArrayRef, EncodingSubclass, PyArray};
+use crate::arrays::native::{AsArrayRef, EncodingSubclass, PyNativeArray};
 use crate::scalar::PyScalar;
 
 /// Concrete class for arrays with `vortex.constant` encoding.
-#[pyclass(name = "ConstantArray", module = "vortex", extends=PyArray, frozen)]
+#[pyclass(name = "ConstantArray", module = "vortex", extends=PyNativeArray, frozen)]
 pub(crate) struct PyConstantArray;
 
 impl EncodingSubclass for PyConstantArray {
@@ -14,11 +14,6 @@ impl EncodingSubclass for PyConstantArray {
 
 #[pymethods]
 impl PyConstantArray {
-    #[new]
-    fn new(array: Bound<PyArray>) -> PyResult<Bound<Self>> {
-        PyArray::init_encoding(array, &ConstantEncoding, PyConstantArray)
-    }
-
     /// Return the scalar value of the constant array.
     pub fn scalar(self_: PyRef<'_, Self>) -> PyResult<Bound<PyScalar>> {
         PyScalar::init(self_.py(), self_.as_array_ref().scalar().clone())
