@@ -1,11 +1,9 @@
-use std::ops::Deref;
-
 use vortex_error::VortexResult;
 
 use crate::Array;
 
 /// Trait for arrays that support iterative access to their elements.
-pub trait ArrayAccessor<Item: ?Sized>: Deref<Target = Array> {
+pub trait ArrayAccessor<Item: ?Sized>: Array {
     /// Iterate over each element of the array, in-order.
     ///
     /// The function `f` will be passed an [`Iterator`], it can call [`next`][Iterator::next] on the
