@@ -3,23 +3,20 @@ use vortex_dtype::DType;
 use vortex_error::{VortexResult, vortex_bail};
 use vortex_scalar::{Scalar, ScalarValue};
 
-use crate::arrays::{ConstantArray, ConstantEncoding};
+use super::ConstantEncoding;
+use crate::arrays::ConstantArray;
+use crate::compute::scalar_at;
 use crate::serde::ArrayParts;
-use crate::vtable::SerdeVTable;
-use crate::{Array, ArrayBufferVisitor, ArrayContext, ArrayRef, ArrayVisitorImpl, EmptyMetadata};
+use crate::vtable::EncodingVTable;
+use crate::{
+    Array, ArrayBufferVisitor, ArrayContext, ArrayRef, ArrayVisitorImpl, EmptyMetadata, EncodingId,
+};
 
-impl ArrayVisitorImpl for ConstantArray {
-    fn _buffers(&self, visitor: &mut dyn ArrayBufferVisitor) {
-        let buffer = self.scalar.value().to_flexbytes::<ByteBufferMut>().freeze();
-        visitor.visit_buffer(&buffer);
+impl EncodingVTable for ConstantEncoding {
+    fn id(&self) -> EncodingId {
+        EncodingId::new_ref("vortex.constant")
     }
 
-    fn _metadata(&self) -> EmptyMetadata {
-        EmptyMetadata
-    }
-}
-
-impl SerdeVTable<&ConstantArray> for ConstantEncoding {
     fn decode(
         &self,
         parts: &ArrayParts,
@@ -34,4 +31,31 @@ impl SerdeVTable<&ConstantArray> for ConstantEncoding {
         let scalar = Scalar::new(dtype, sv);
         Ok(ConstantArray::new(scalar, len).into_array())
     }
+
+    fn encode(
+        &self,
+        input: &crate::Canonical,
+        _like: Option<&dyn Array>,
+    ) -> VortexResult<Option<ArrayRef>> {
+        let array_ref = input.as_ref();
+        if array_ref.is_constant() {
+            let scalar = scalar_at(array_ref, 0)?;
+            Ok(Some(
+                ConstantArray::new(scalar, array_ref.len()).into_array(),
+            ))
+        } else {
+            Ok(None)
+        }
+    }
+}
+
+impl ArrayVisitorImpl for ConstantArray {
+    fn _visit_buffers(&self, visitor: &mut dyn ArrayBufferVisitor) {
+        let buffer = self.scalar.value().to_flexbytes::<ByteBufferMut>().freeze();
+        visitor.visit_buffer(&buffer);
+    }
+
+    fn _metadata(&self) -> EmptyMetadata {
+        EmptyMetadata
+    }
 }
