@@ -11,7 +11,9 @@ use arrow_buffer::ScalarBuffer;
 use static_assertions::{assert_eq_align, assert_eq_size};
 use vortex_buffer::{Alignment, Buffer, ByteBuffer};
 use vortex_dtype::DType;
-use vortex_error::{VortexExpect, VortexResult, VortexUnwrap, vortex_bail, vortex_panic};
+use vortex_error::{
+    VortexExpect, VortexResult, VortexUnwrap, vortex_bail, vortex_err, vortex_panic,
+};
 use vortex_mask::Mask;
 
 use crate::array::{ArrayCanonicalImpl, ArrayValidityImpl};
@@ -19,16 +21,15 @@ use crate::arrow::FromArrowArray;
 use crate::builders::ArrayBuilder;
 use crate::stats::{ArrayStats, StatsSetRef};
 use crate::validity::Validity;
-use crate::vtable::{EncodingVTable, VTableRef};
+use crate::vtable::VTableRef;
 use crate::{
     Array, ArrayImpl, ArrayRef, ArrayStatisticsImpl, Canonical, EmptyMetadata, Encoding,
-    EncodingId, TryFromArrayRef, try_from_array_ref,
+    TryFromArrayRef, try_from_array_ref,
 };
 
 mod accessor;
 mod compute;
 mod serde;
-mod stats;
 mod variants;
 
 #[derive(Clone, Copy, Debug, PartialEq, Eq)]
@@ -39,17 +40,12 @@ pub struct Inlined {
 }
 
 impl Inlined {
-    pub fn new(value: &[u8]) -> Self {
-        assert!(
-            value.len() <= BinaryView::MAX_INLINED_SIZE,
-            "Inlined strings must be shorter than 13 characters, {} given",
-            value.len()
-        );
+    fn new<const N: usize>(value: &[u8]) -> Self {
         let mut inlined = Self {
-            size: value.len().try_into().vortex_unwrap(),
+            size: N.try_into().vortex_unwrap(),
             data: [0u8; BinaryView::MAX_INLINED_SIZE],
         };
-        inlined.data[..value.len()].copy_from_slice(value);
+        inlined.data[..N].copy_from_slice(&value[..N]);
         inlined
     }
 
@@ -121,12 +117,74 @@ assert_eq_align!(BinaryView, u128);
 impl BinaryView {
     pub const MAX_INLINED_SIZE: usize = 12;
 
-    pub fn empty_view() -> Self {
-        Self {
-            inlined: Inlined::new(&[]),
+    /// Create a view from a value, block and offset
+    ///
+    /// Depending on the length of the provided value either a new inlined
+    /// or a reference view will be constructed.
+    ///
+    /// Adapted from arrow-rs <https://github.com/apache/arrow-rs/blob/f4fde769ab6e1a9b75f890b7f8b47bc22800830b/arrow-array/src/builder/generic_bytes_view_builder.rs#L524>
+    /// Explicitly enumerating inlined view produces code that avoids calling generic `ptr::copy_non_interleave` that's slower than explicit stores
+    #[inline(never)]
+    pub fn make_view(value: &[u8], block: u32, offset: u32) -> Self {
+        match value.len() {
+            0 => Self {
+                inlined: Inlined::new::<0>(value),
+            },
+            1 => Self {
+                inlined: Inlined::new::<1>(value),
+            },
+            2 => Self {
+                inlined: Inlined::new::<2>(value),
+            },
+            3 => Self {
+                inlined: Inlined::new::<3>(value),
+            },
+            4 => Self {
+                inlined: Inlined::new::<4>(value),
+            },
+            5 => Self {
+                inlined: Inlined::new::<5>(value),
+            },
+            6 => Self {
+                inlined: Inlined::new::<6>(value),
+            },
+            7 => Self {
+                inlined: Inlined::new::<7>(value),
+            },
+            8 => Self {
+                inlined: Inlined::new::<8>(value),
+            },
+            9 => Self {
+                inlined: Inlined::new::<9>(value),
+            },
+            10 => Self {
+                inlined: Inlined::new::<10>(value),
+            },
+            11 => Self {
+                inlined: Inlined::new::<11>(value),
+            },
+            12 => Self {
+                inlined: Inlined::new::<12>(value),
+            },
+            _ => Self {
+                _ref: Ref::new(
+                    u32::try_from(value.len()).vortex_unwrap(),
+                    value[0..4].try_into().vortex_unwrap(),
+                    block,
+                    offset,
+                ),
+            },
         }
     }
 
+    /// Create a new empty view
+    #[inline]
+    pub fn empty_view() -> Self {
+        Self::new_inlined(&[])
+    }
+
+    /// Create a new inlined binary view
+    #[inline]
     pub fn new_inlined(value: &[u8]) -> Self {
         assert!(
             value.len() <= Self::MAX_INLINED_SIZE,
@@ -134,16 +192,7 @@ impl BinaryView {
             value.len()
         );
 
-        Self {
-            inlined: Inlined::new(value),
-        }
-    }
-
-    /// Create a new view over bytes stored in a block.
-    pub fn new_view(len: u32, prefix: [u8; 4], block: u32, offset: u32) -> Self {
-        Self {
-            _ref: Ref::new(len, prefix, block, offset),
-        }
+        Self::make_view(value, 0, 0)
     }
 
     #[inline]
@@ -184,12 +233,14 @@ impl BinaryView {
         } else {
             // Referencing views must have their buffer_index adjusted with new offsets
             let view_ref = self.as_view();
-            BinaryView::new_view(
-                self.len(),
-                *view_ref.prefix(),
-                offset + view_ref.buffer_index(),
-                view_ref.offset(),
-            )
+            Self {
+                _ref: Ref::new(
+                    self.len(),
+                    *view_ref.prefix(),
+                    offset + view_ref.buffer_index(),
+                    view_ref.offset(),
+                ),
+            }
         }
     }
 }
@@ -231,12 +282,6 @@ impl Encoding for VarBinViewEncoding {
     type Metadata = EmptyMetadata;
 }
 
-impl EncodingVTable for VarBinViewEncoding {
-    fn id(&self) -> EncodingId {
-        EncodingId::new_ref("vortex.varbinview")
-    }
-}
-
 impl VarBinViewArray {
     pub fn try_new(
         views: Buffer<BinaryView>,
@@ -354,6 +399,7 @@ impl VarBinViewArray {
                     &string_view_array,
                     nullability.into(),
                 ))
+                .map_err(|_| vortex_err!("Array was not a VarBinViewArray"))
                 .vortex_expect("StringViewArray to VarBinViewArray downcast")
             }
             DType::Binary(nullability) => {
@@ -365,6 +411,7 @@ impl VarBinViewArray {
                     &binary_view_array,
                     nullability.into(),
                 ))
+                .map_err(|_| vortex_err!("Array was not a VarBinViewArray"))
                 .vortex_expect("BinaryViewArray to VarBinViewArray downcast")
             }
             other => vortex_panic!("VarBinViewArray must be Utf8 or Binary, was {other}"),
@@ -379,6 +426,7 @@ impl VarBinViewArray {
         }
         let array = ArrayRef::from_arrow(&builder.finish(), false);
         VarBinViewArray::try_from_array(array)
+            .map_err(|_| vortex_err!("Array was not a VarBinViewArray"))
             .vortex_expect("VarBinViewArray from StringViewBuilder")
     }
 
@@ -391,6 +439,7 @@ impl VarBinViewArray {
 
         let array = ArrayRef::from_arrow(&builder.finish(), true);
         VarBinViewArray::try_from_array(array)
+            .map_err(|_| vortex_err!("Array was not a VarBinViewArray"))
             .vortex_expect("VarBinViewArray from StringViewBuilder")
     }
 
@@ -402,6 +451,7 @@ impl VarBinViewArray {
         }
         let array = ArrayRef::from_arrow(&builder.finish(), false);
         VarBinViewArray::try_from_array(array)
+            .map_err(|_| vortex_err!("Array was not a VarBinViewArray"))
             .vortex_expect("VarBinViewArray from StringViewBuilder")
     }
 
@@ -413,6 +463,7 @@ impl VarBinViewArray {
         builder.extend(iter);
         let array = ArrayRef::from_arrow(&builder.finish(), true);
         VarBinViewArray::try_from_array(array)
+            .map_err(|_| vortex_err!("Array was not a VarBinViewArray"))
             .vortex_expect("VarBinViewArray from StringViewBuilder")
     }
 }
@@ -450,6 +501,16 @@ impl ArrayImpl for VarBinViewArray {
     fn _vtable(&self) -> VTableRef {
         VTableRef::new_ref(&VarBinViewEncoding)
     }
+
+    fn _with_children(&self, children: &[ArrayRef]) -> VortexResult<Self> {
+        let mut this = self.clone();
+
+        if let Validity::Array(array) = &mut this.validity {
+            *array = children[0].clone();
+        }
+
+        Ok(this)
+    }
 }
 
 impl ArrayStatisticsImpl for VarBinViewArray {
@@ -519,7 +580,7 @@ impl ArrayValidityImpl for VarBinViewArray {
     }
 
     fn _validity_mask(&self) -> VortexResult<Mask> {
-        self.validity.to_logical(self.len())
+        self.validity.to_mask(self.len())
     }
 }
 
