@@ -161,6 +161,10 @@ impl Patches {
         self.indices
     }
 
+    pub fn indices_mut(&mut self) -> &mut ArrayRef {
+        &mut self.indices
+    }
+
     pub fn values(&self) -> &ArrayRef {
         &self.values
     }
@@ -169,6 +173,10 @@ impl Patches {
         self.values
     }
 
+    pub fn values_mut(&mut self) -> &mut ArrayRef {
+        &mut self.values
+    }
+
     pub fn offset(&self) -> usize {
         self.offset
     }
@@ -217,8 +225,8 @@ impl Patches {
         }
     }
 
-    /// Return the insertion point of [index] in the [Self::indices].
-    fn search_index(&self, index: usize) -> VortexResult<SearchResult> {
+    /// Return the insertion point of `index` in the [Self::indices].
+    pub fn search_index(&self, index: usize) -> VortexResult<SearchResult> {
         search_sorted_usize(&self.indices, index + self.offset, SearchSortedSide::Left)
     }
 
@@ -229,15 +237,17 @@ impl Patches {
         side: SearchSortedSide,
     ) -> VortexResult<SearchResult> {
         search_sorted(self.values(), target.into(), side).and_then(|sr| {
-            let sidx = sr.to_offsets_index(self.indices().len());
-            let index = usize::try_from(&scalar_at(self.indices(), sidx)?)? - self.offset;
+            let index_idx = sr.to_offsets_index(self.indices().len(), side);
+            let index = usize::try_from(&scalar_at(self.indices(), index_idx)?)? - self.offset;
             Ok(match sr {
                 // If we reached the end of patched values when searching then the result is one after the last patch index
-                SearchResult::Found(i) => SearchResult::Found(if i == self.indices().len() {
-                    index + 1
-                } else {
-                    index
-                }),
+                SearchResult::Found(i) => SearchResult::Found(
+                    if i == self.indices().len() || side == SearchSortedSide::Right {
+                        index + 1
+                    } else {
+                        index
+                    },
+                ),
                 // If the result is NotFound we should return index that's one after the nearest not found index for the corresponding value
                 SearchResult::NotFound(i) => {
                     SearchResult::NotFound(if i == 0 { index } else { index + 1 })
@@ -636,19 +646,45 @@ mod test {
     #[test]
     fn search_right() {
         let patches = Patches::new(
-            2,
+            6,
             0,
-            buffer![0u64].into_array(),
-            PrimitiveArray::new(buffer![0u8], Validity::AllValid).into_array(),
+            buffer![0u8, 1, 4, 5].into_array(),
+            buffer![-128i8, -98, 8, 50].into_array(),
         );
 
         assert_eq!(
-            patches.search_sorted(0, SearchSortedSide::Right).unwrap(),
-            SearchResult::Found(1)
+            patches.search_sorted(-98, SearchSortedSide::Right).unwrap(),
+            SearchResult::Found(2)
         );
         assert_eq!(
-            patches.search_sorted(1, SearchSortedSide::Right).unwrap(),
-            SearchResult::NotFound(1)
+            patches.search_sorted(50, SearchSortedSide::Right).unwrap(),
+            SearchResult::Found(6),
+        );
+        assert_eq!(
+            patches.search_sorted(7, SearchSortedSide::Right).unwrap(),
+            SearchResult::NotFound(2),
+        );
+        assert_eq!(
+            patches.search_sorted(51, SearchSortedSide::Right).unwrap(),
+            SearchResult::NotFound(6)
+        );
+    }
+
+    #[test]
+    fn search_left() {
+        let patches = Patches::new(
+            20,
+            0,
+            buffer![0u64, 1, 17, 18, 19].into_array(),
+            buffer![11i32, 22, 33, 44, 55].into_array(),
+        );
+        assert_eq!(
+            patches.search_sorted(30, SearchSortedSide::Left).unwrap(),
+            SearchResult::NotFound(2)
+        );
+        assert_eq!(
+            patches.search_sorted(54, SearchSortedSide::Left).unwrap(),
+            SearchResult::NotFound(19)
         );
     }
 
@@ -669,4 +705,37 @@ mod test {
             Mask::from_iter(vec![true])
         );
     }
+
+    #[test]
+    fn test_slice() {
+        let values = buffer![15_u32, 135, 13531, 42].into_array();
+        let indices = buffer![10_u64, 11, 50, 100].into_array();
+
+        let patches = Patches::new(101, 0, indices, values);
+
+        let sliced = patches.slice(15, 100).unwrap().unwrap();
+        assert_eq!(sliced.array_len(), 100 - 15);
+        let primitive = sliced.values().to_primitive().unwrap();
+
+        assert_eq!(primitive.as_slice::<u32>(), &[13531]);
+    }
+
+    #[test]
+    fn doubly_sliced() {
+        let values = buffer![15_u32, 135, 13531, 42].into_array();
+        let indices = buffer![10_u64, 11, 50, 100].into_array();
+
+        let patches = Patches::new(101, 0, indices, values);
+
+        let sliced = patches.slice(15, 100).unwrap().unwrap();
+        assert_eq!(sliced.array_len(), 100 - 15);
+        let primitive = sliced.values().to_primitive().unwrap();
+
+        assert_eq!(primitive.as_slice::<u32>(), &[13531]);
+
+        let doubly_sliced = sliced.slice(35, 36).unwrap().unwrap();
+        let primitive_doubly_sliced = doubly_sliced.values().to_primitive().unwrap();
+
+        assert_eq!(primitive_doubly_sliced.as_slice::<u32>(), &[13531]);
+    }
 }
