@@ -1,8 +1,9 @@
 use crate::Array;
 use crate::compute::{
     BetweenFn, BinaryBooleanFn, BinaryNumericFn, CastFn, CompareFn, FillForwardFn, FillNullFn,
-    FilterFn, InvertFn, IsConstantFn, LikeFn, MaskFn, MinMaxFn, ScalarAtFn, SearchSortedFn,
-    SearchSortedUsizeFn, SliceFn, SumFn, TakeFn, TakeFromFn, ToArrowFn, UncompressedSizeFn,
+    InvertFn, IsConstantFn, IsSortedFn, LikeFn, MaskFn, MinMaxFn, OptimizeFn, ScalarAtFn,
+    SearchSortedFn, SearchSortedUsizeFn, SliceFn, SumFn, TakeFn, TakeFromFn, ToArrowFn,
+    UncompressedSizeFn,
 };
 
 /// VTable for dispatching compute functions to Vortex encodings.
@@ -56,13 +57,6 @@ pub trait ComputeVTable {
         None
     }
 
-    /// Filter an array with a given mask.
-    ///
-    /// See: [`FilterFn`].
-    fn filter_fn(&self) -> Option<&dyn FilterFn<&dyn Array>> {
-        None
-    }
-
     /// Invert a boolean array. Converts true -> false, false -> true, null -> null.
     ///
     /// See [`InvertFn`]
@@ -77,6 +71,10 @@ pub trait ComputeVTable {
         None
     }
 
+    fn is_sorted_fn(&self) -> Option<&dyn IsSortedFn<&dyn Array>> {
+        None
+    }
+
     /// Perform a SQL LIKE operation on two arrays.
     ///
     /// See: [`LikeFn`].
@@ -94,10 +92,19 @@ pub trait ComputeVTable {
     }
 
     /// Compute the min, max of an array.
+    ///
+    /// See: [`MinMaxFn`].
     fn min_max_fn(&self) -> Option<&dyn MinMaxFn<&dyn Array>> {
         None
     }
 
+    /// Try and optimize the layout of an array.
+    ///
+    /// See: [`OptimizeFn`]
+    fn optimize_fn(&self) -> Option<&dyn OptimizeFn<&dyn Array>> {
+        None
+    }
+
     /// Single item indexing on Vortex arrays.
     ///
     /// See: [`ScalarAtFn`].
