@@ -4,16 +4,15 @@ use std::fmt::{Debug, Display, Formatter};
 use std::hash::{Hash, Hasher};
 
 mod compute;
-mod serde;
-mod statistics;
 
 pub use compute::*;
-pub use serde::*;
-pub use statistics::*;
+use vortex_dtype::DType;
+use vortex_error::{VortexExpect, VortexResult, vortex_bail};
 
-use crate::Array;
 use crate::arcref::ArcRef;
 use crate::encoding::EncodingId;
+use crate::serde::ArrayParts;
+use crate::{Array, ArrayContext, ArrayRef, Canonical};
 
 /// A reference to an array VTable, either static or arc'd.
 pub type VTableRef = ArcRef<dyn EncodingVTable>;
@@ -24,16 +23,39 @@ pub type VTableRef = ArcRef<dyn EncodingVTable>;
 /// It is split into multiple sub-traits to make it easier for consumers to break up the
 /// implementation, as well as to allow for optional implementation of certain features, for example
 /// compute functions.
-pub trait EncodingVTable:
-    'static
-    + Sync
-    + Send
-    + ComputeVTable
-    + for<'a> SerdeVTable<&'a dyn Array>
-    + for<'a> StatisticsVTable<&'a dyn Array>
-{
+pub trait EncodingVTable: 'static + Sync + Send + ComputeVTable {
     /// Return the ID for this encoding implementation.
     fn id(&self) -> EncodingId;
+
+    fn decode(
+        &self,
+        parts: &ArrayParts,
+        ctx: &ArrayContext,
+        _dtype: DType,
+        _len: usize,
+    ) -> VortexResult<ArrayRef> {
+        vortex_bail!(
+            "Decoding not supported for encoding {}",
+            ctx.lookup_encoding(parts.encoding_id())
+                .vortex_expect("Encoding already validated")
+                .id()
+        )
+    }
+
+    /// Encode the canonical array into this encoding implementation.
+    ///
+    /// Should error if `like` is encoded with a different encoding.
+    fn encode(
+        &self,
+        input: &Canonical,
+        _like: Option<&dyn Array>,
+    ) -> VortexResult<Option<ArrayRef>> {
+        if self.id() == input.as_ref().encoding() {
+            return Ok(Some(input.as_ref().to_array()));
+        }
+
+        Ok(None)
+    }
 }
 
 impl PartialEq for dyn EncodingVTable + '_ {
