@@ -1,11 +1,11 @@
 //! Specialized compressor for DateTimeParts metadata.
 
 use vortex_array::arrays::TemporalArray;
+use vortex_array::compress::downscale_integer_array;
 use vortex_array::{Array, ArrayRef, ToCanonical};
 use vortex_datetime_parts::{DateTimePartsArray, TemporalParts, split_temporal};
 use vortex_error::VortexResult;
 
-use crate::downscale::downscale_integer_array;
 use crate::integer::IntCompressor;
 use crate::{Compressor, MAX_CASCADE};
 
