@@ -6,7 +6,7 @@ use std::hash::{Hash, Hasher};
 use std::ops::{Deref, RangeBounds};
 
 use bytes::{Buf, Bytes};
-use vortex_error::{vortex_panic, VortexExpect};
+use vortex_error::{VortexExpect, vortex_panic};
 
 use crate::debug::TruncatedDebug;
 use crate::{Alignment, BufferMut, ByteBuffer};
@@ -308,6 +308,16 @@ impl<T> Buffer<T> {
         }
     }
 
+    /// Returns the underlying aligned buffer.
+    pub fn inner(&self) -> &Bytes {
+        debug_assert_eq!(
+            self.length * size_of::<T>(),
+            self.bytes.len(),
+            "Own length has to be the same as the underlying bytes length"
+        );
+        &self.bytes
+    }
+
     /// Returns the underlying aligned buffer.
     pub fn into_inner(self) -> Bytes {
         debug_assert_eq!(
@@ -352,6 +362,11 @@ impl<T> Buffer<T> {
             })
     }
 
+    /// Returns whether a `Buffer<T>` is aligned to the given alignment.
+    pub fn is_aligned(&self, alignment: Alignment) -> bool {
+        self.bytes.as_ptr().align_offset(*alignment) == 0
+    }
+
     /// Return a `Buffer<T>` with the given alignment. Where possible, this will be zero-copy.
     pub fn aligned(mut self, alignment: Alignment) -> Self {
         if self.as_ptr().align_offset(*alignment) == 0 {
@@ -454,7 +469,7 @@ impl Buf for ByteBuffer {
     }
 }
 
-/// Owned iterator over a `Buffer<T>`.
+/// Owned iterator over a [`Buffer`].
 pub struct BufferIterator<T> {
     buffer: Buffer<T>,
     index: usize,
@@ -499,7 +514,7 @@ impl<T> From<BufferMut<T>> for Buffer<T> {
 mod test {
     use bytes::Buf;
 
-    use crate::{buffer, Alignment, ByteBuffer};
+    use crate::{Alignment, ByteBuffer, buffer};
 
     #[test]
     fn align() {
