@@ -25,6 +25,18 @@ impl From<&str> for Field {
     }
 }
 
+impl From<Arc<str>> for Field {
+    fn from(value: Arc<str>) -> Self {
+        Self::Name(value)
+    }
+}
+
+impl From<&Arc<str>> for Field {
+    fn from(value: &Arc<str>) -> Self {
+        Self::Name(value.clone())
+    }
+}
+
 impl From<String> for Field {
     fn from(value: String) -> Self {
         Field::Name(value.into())
@@ -48,7 +60,6 @@ impl Field {
 }
 
 /// A path through a (possibly nested) struct, composed of a sequence of field selectors
-// TODO(ngates): wrap `Vec<Field>` in Option for cheaper "root" path.
 // TODO(ngates): we should probably reverse the path. Or better yet, store a Arc<[Field]> along
 //  with a positional index to allow cheap step_into.
 #[derive(Clone, Debug, PartialEq, Eq, Hash)]
