@@ -16,7 +16,7 @@ use std::error::Error;
 use std::fmt::{Debug, Display, Formatter};
 use std::num::TryFromIntError;
 use std::ops::Deref;
-use std::sync::PoisonError;
+use std::sync::{Arc, PoisonError};
 use std::{env, fmt, io};
 
 pub use ext::*;
@@ -100,13 +100,12 @@ pub enum VortexError {
     /// A wrapper for other errors, carrying additional context.
     #[error("{0}: {1}")]
     Context(ErrString, #[source] Box<VortexError>),
-    /// A wrapper for errors from the Arrow library.
+    /// A wrapper for shared errors that require cloning.
     #[error(transparent)]
-    ArrowError(
-        #[from]
-        #[backtrace]
-        arrow_schema::ArrowError,
-    ),
+    Shared(Arc<VortexError>),
+    /// A wrapper for errors from the Arrow library.
+    #[error("{0}\nBacktrace:\n{1}")]
+    ArrowError(arrow_schema::ArrowError, Backtrace),
     /// A wrapper for errors from the FlatBuffers library.
     #[cfg(feature = "flatbuffers")]
     #[error(transparent)]
@@ -228,6 +227,38 @@ pub enum VortexError {
         #[backtrace]
         TryFromIntError,
     ),
+    /// Wrap serde and serde json errors
+    #[cfg(feature = "serde")]
+    #[error(transparent)]
+    SerdeJsonError(
+        #[from]
+        #[backtrace]
+        serde_json::Error,
+    ),
+    /// Wrap prost encode error
+    #[cfg(feature = "prost")]
+    #[error(transparent)]
+    ProstEncodeError(
+        #[from]
+        #[backtrace]
+        prost::EncodeError,
+    ),
+    /// Wrap prost decode error
+    #[cfg(feature = "prost")]
+    #[error(transparent)]
+    ProstDecodeError(
+        #[from]
+        #[backtrace]
+        prost::DecodeError,
+    ),
+    /// Wrap prost unknown enum value
+    #[cfg(feature = "prost")]
+    #[error(transparent)]
+    ProstUnknownEnumValue(
+        #[from]
+        #[backtrace]
+        prost::UnknownEnumValue,
+    ),
 }
 
 impl VortexError {
@@ -246,6 +277,26 @@ impl Debug for VortexError {
 /// A type alias for Results that return VortexErrors as their error type.
 pub type VortexResult<T> = Result<T, VortexError>;
 
+/// A vortex result that can be shared or cloned.
+pub type SharedVortexResult<T> = Result<T, Arc<VortexError>>;
+
+impl From<Arc<VortexError>> for VortexError {
+    fn from(value: Arc<VortexError>) -> Self {
+        Self::from(&value)
+    }
+}
+
+impl From<&Arc<VortexError>> for VortexError {
+    fn from(e: &Arc<VortexError>) -> Self {
+        if let VortexError::Shared(e_inner) = e.as_ref() {
+            // don't re-wrap
+            VortexError::Shared(Arc::clone(e_inner))
+        } else {
+            VortexError::Shared(Arc::clone(e))
+        }
+    }
+}
+
 /// A trait for unwrapping a VortexResult.
 pub trait VortexUnwrap {
     /// The type of the value being unwrapped.
@@ -307,6 +358,13 @@ impl<T> VortexExpect for Option<T> {
 /// A convenient macro for creating a VortexError.
 #[macro_export]
 macro_rules! vortex_err {
+    (AssertionFailed: $($tts:tt)*) => {{
+        use std::backtrace::Backtrace;
+        let err_string = format!($($tts)*);
+        $crate::__private::must_use(
+            $crate::VortexError::AssertionFailed(err_string.into(), Backtrace::capture())
+        )
+    }};
     (OutOfBounds: $idx:expr, $start:expr, $stop:expr) => {{
         use std::backtrace::Backtrace;
         $crate::__private::must_use(
@@ -395,6 +453,12 @@ macro_rules! vortex_panic {
     }};
 }
 
+impl From<arrow_schema::ArrowError> for VortexError {
+    fn from(value: arrow_schema::ArrowError) -> Self {
+        VortexError::ArrowError(value, Backtrace::capture())
+    }
+}
+
 #[cfg(feature = "datafusion")]
 impl From<VortexError> for datafusion_common::DataFusionError {
     fn from(value: VortexError) -> Self {
@@ -406,7 +470,7 @@ impl From<VortexError> for datafusion_common::DataFusionError {
 impl From<VortexError> for datafusion_common::arrow::error::ArrowError {
     fn from(value: VortexError) -> Self {
         match value {
-            VortexError::ArrowError(e) => e,
+            VortexError::ArrowError(e, _) => e,
             _ => Self::from_external_error(Box::new(value)),
         }
     }
