@@ -166,10 +166,12 @@ mod test {
             .collect_to_buffer()
             .unwrap();
 
-        let reader = SyncIPCReader::try_new(Cursor::new(ipc_buffer), Default::default()).unwrap();
+        let reader =
+            SyncIPCReader::try_new(Cursor::new(ipc_buffer), ArrayRegistry::canonical_only())
+                .unwrap();
 
         assert_eq!(reader.dtype(), array.dtype());
-        let result = reader.into_array().unwrap().to_primitive().unwrap();
+        let result = reader.read_all().unwrap().to_primitive().unwrap();
         assert_eq!(array.as_slice::<i32>(), result.as_slice::<i32>());
     }
 }
