@@ -11,10 +11,3 @@
 pub mod iterator;
 pub mod messages;
 pub mod stream;
-
-/// All messages in Vortex are aligned to start at a multiple of 64 bytes.
-///
-/// This is a multiple of the native alignment for all PTypes,
-/// thus all buffers allocated with this alignment are naturally aligned
-/// for any data we may put inside of it.
-pub const ALIGNMENT: usize = 64;
