@@ -1,5 +1,8 @@
 //! A collection of built-in layouts for Vortex
 pub mod chunked;
+pub mod file_stats;
+pub mod filter;
 pub mod flat;
+pub mod repartition;
 pub mod stats;
 pub mod struct_;
