@@ -1,4 +1,5 @@
 use std::iter;
+use std::ops::Range;
 use std::sync::Arc;
 
 use futures::executor::LocalPool;
@@ -32,6 +33,9 @@ pub struct ScanBuilder {
     layout_reader: Arc<dyn LayoutReader>,
     projection: ExprRef,
     filter: Option<ExprRef>,
+    /// Optionally read a subset of the rows in the file.
+    row_range: Option<Range<u64>>,
+    /// The selection mask to apply to the selected row range.
     selection: Selection,
     /// How to split the file for concurrent processing.
     split_by: SplitBy,
@@ -48,6 +52,7 @@ impl ScanBuilder {
             layout_reader,
             projection: Identity::new_expr(),
             filter: None,
+            row_range: None,
             selection: Default::default(),
             split_by: SplitBy::Layout,
             canonicalize: false,
@@ -73,6 +78,16 @@ impl ScanBuilder {
         self
     }
 
+    pub fn with_row_range(mut self, row_range: Range<u64>) -> Self {
+        self.row_range = Some(row_range);
+        self
+    }
+
+    pub fn with_some_row_range(mut self, row_range: Option<Range<u64>>) -> Self {
+        self.row_range = row_range;
+        self
+    }
+
     pub fn with_selection(mut self, selection: Selection) -> Self {
         self.selection = selection;
         self
@@ -137,6 +152,20 @@ impl ScanBuilder {
 
         let row_masks = splits
             .into_iter()
+            .filter_map(|row_range| {
+                if let Some(scan_range) = &self.row_range {
+                    // If the row range is fully within the scan range, return it.
+                    if row_range.start >= scan_range.end || row_range.end < scan_range.start {
+                        return None;
+                    }
+                    // Otherwise, take the intersection of the range.
+                    return Some(
+                        row_range.start.max(scan_range.start)..row_range.end.min(scan_range.end),
+                    );
+                } else {
+                    Some(row_range)
+                }
+            })
             .map(|row_range| self.selection.row_mask(&row_range))
             .filter(|mask| !mask.mask().all_false())
             .collect_vec();
