@@ -1,3 +1,4 @@
+use std::fmt::{Display, Formatter};
 use std::hash::Hash;
 use std::ops::Deref;
 use std::sync::Arc;
@@ -16,6 +17,24 @@ pub struct ListScalar<'a> {
     elements: Option<Arc<[ScalarValue]>>,
 }
 
+impl Display for ListScalar<'_> {
+    fn fmt(&self, f: &mut Formatter<'_>) -> std::fmt::Result {
+        match &self.elements {
+            None => write!(f, "null"),
+            Some(elems) => {
+                write!(
+                    f,
+                    "[{}]",
+                    elems
+                        .iter()
+                        .map(|e| Scalar::new(self.element_dtype().clone(), e.clone()))
+                        .format(", ")
+                )
+            }
+        }
+    }
+}
+
 impl PartialEq for ListScalar<'_> {
     fn eq(&self, other: &Self) -> bool {
         self.dtype.eq_ignore_nullability(other.dtype) && self.elements() == other.elements()
