@@ -1,6 +1,6 @@
 use std::any::type_name;
 use std::cmp::Ordering;
-use std::fmt::{Debug, Display};
+use std::fmt::{Debug, Display, Formatter};
 use std::ops::{Add, Sub};
 
 use num_traits::{CheckedAdd, CheckedDiv, CheckedMul, CheckedSub, FromPrimitive};
@@ -20,6 +20,15 @@ pub struct PrimitiveScalar<'a> {
     pvalue: Option<PValue>,
 }
 
+impl Display for PrimitiveScalar<'_> {
+    fn fmt(&self, f: &mut Formatter<'_>) -> std::fmt::Result {
+        match self.pvalue {
+            None => write!(f, "null"),
+            Some(pv) => write!(f, "{}", pv),
+        }
+    }
+}
+
 impl PartialEq for PrimitiveScalar<'_> {
     fn eq(&self, other: &Self) -> bool {
         self.dtype.eq_ignore_nullability(other.dtype) && self.pvalue == other.pvalue
@@ -354,7 +363,7 @@ pub enum BinaryNumericOperator {
 }
 
 impl Display for BinaryNumericOperator {
-    fn fmt(&self, f: &mut std::fmt::Formatter<'_>) -> std::fmt::Result {
+    fn fmt(&self, f: &mut Formatter<'_>) -> std::fmt::Result {
         Debug::fmt(self, f)
     }
 }
