@@ -1,3 +1,4 @@
+use std::fmt::{Display, Formatter};
 use std::sync::Arc;
 
 use vortex_buffer::BufferString;
@@ -13,6 +14,15 @@ pub struct Utf8Scalar<'a> {
     value: Option<BufferString>,
 }
 
+impl Display for Utf8Scalar<'_> {
+    fn fmt(&self, f: &mut Formatter<'_>) -> std::fmt::Result {
+        match &self.value {
+            None => write!(f, "null"),
+            Some(v) => write!(f, "\"{}\"", v.as_str()),
+        }
+    }
+}
+
 impl PartialEq for Utf8Scalar<'_> {
     fn eq(&self, other: &Self) -> bool {
         self.dtype.eq_ignore_nullability(other.dtype) && self.value == other.value
